@@ -26,6 +26,8 @@
 #include <OpenImageIO/platform.h>
 #include <OpenImageIO/string_view.h>
 
+#include <OpenImageIO/detail/farmhash.h>
+
 #if OIIO_GNUC_VERSION >= 70000
 #    pragma GCC diagnostic push
 #    pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
@@ -59,24 +61,6 @@
 // was locale dependent.
 #define OIIO_FMT_LOCALE_INDEPENDENT 1
 
-#ifndef OPENIMAGEIO_PRINTF_ARGS
-#   ifndef __GNUC__
-#       define __attribute__(x)
-#   endif
-    // Enable printf-like warnings with gcc by attaching
-    // OPENIMAGEIO_PRINTF_ARGS to printf-like functions.  Eg:
-    //
-    // void foo (const char* fmt, ...) OPENIMAGEIO_PRINTF_ARGS(1,2);
-    //
-    // The arguments specify the positions of the format string and the the
-    // beginning of the varargs parameter list respectively.
-    //
-    // For member functions with arguments like the example above, you need
-    // OPENIMAGEIO_PRINTF_ARGS(2,3) instead.  (gcc includes the implicit this
-    // pointer when it counts member function arguments.)
-#   define OPENIMAGEIO_PRINTF_ARGS(fmtarg_pos, vararg_pos) \
-        __attribute__ ((format (printf, fmtarg_pos, vararg_pos) ))
-#endif
 
 
 OIIO_NAMESPACE_BEGIN
@@ -231,14 +215,20 @@ inline void print (std::ostream &file, const char* fmt, const Args&... args)
 /// already as a va_list.  This is not guaranteed type-safe and is not
 /// extensible like format(). Use with caution!
 std::string OIIO_API vsprintf (const char *fmt, va_list ap)
-                                         OPENIMAGEIO_PRINTF_ARGS(1,0);
+#if defined(__GNUC__) && !defined(__CUDACC__)
+    __attribute__ ((format (printf, 1, 0) ))
+#endif
+    ;
 
 /// Return a std::string formatted like Strutil::format, but passed
 /// already as a va_list.  This is not guaranteed type-safe and is not
 /// extensible like format(). Use with caution!
 OIIO_DEPRECATED("use `vsprintf` instead")
 std::string OIIO_API vformat (const char *fmt, va_list ap)
-                                         OPENIMAGEIO_PRINTF_ARGS(1,0);
+#if defined(__GNUC__) && !defined(__CUDACC__)
+    __attribute__ ((format (printf, 1, 0) ))
+#endif
+    ;
 
 /// Return a string expressing a number of bytes, in human readable form.
 ///  - memformat(153)           -> "153 B"
@@ -288,11 +278,25 @@ std::string OIIO_API wordwrap (string_view src, int columns = 80,
                                int prefix = 0, string_view sep = " ",
                                string_view presep = "");
 
-/// Hash a string_view.
-inline size_t
+
+/// Our favorite "string" hash of a length of bytes. Currently, it is just
+/// a wrapper for an inlined, constexpr (if C++ >= 14), Cuda-safe farmhash.
+inline OIIO_CONSTEXPR14 size_t
+strhash (size_t len, const char *s)
+{
+    return OIIO::farmhash::inlined::Hash(s, len);
+}
+
+
+/// Hash a string_view. This is OIIO's default favorite string hasher.
+/// Currently, it uses farmhash, is constexpr (for C++14), and works in
+/// Cuda. This is rigged, though, so that empty strings hash always hash to
+/// 0 (that isn't would a raw farmhash would give you, but it's a useful
+/// property, especially for trivial initialization).
+inline OIIO_CONSTEXPR14 size_t
 strhash (string_view s)
 {
-    return s.length() ? farmhash::Hash (s) : 0;
+    return s.length() ? strhash(s.length(), s.data()) : 0;
 }
 
 
@@ -322,6 +326,22 @@ bool OIIO_API iends_with (string_view a, string_view b);
 /// Does 'a' contain the string 'b' within it?
 bool OIIO_API contains (string_view a, string_view b);
 
+/// Return the position of the first occurrence of `b` within `a`, or
+/// std::npos if not found.
+size_t OIIO_API find(string_view a, string_view b);
+
+/// Return the position of the first occurrence of `b` within `a`, with a
+/// case-insensitive comparison, or std::npos if not found.
+size_t OIIO_API ifind(string_view a, string_view b);
+
+/// Return the position of the last occurrence of `b` within `a`, or npos if
+/// not found.
+size_t OIIO_API rfind(string_view a, string_view b);
+
+/// Return the position of the last occurrence of `b` within `a`, with a
+/// case-insensitive comparison, or npos if not found.
+size_t OIIO_API irfind(string_view a, string_view b);
+
 /// Does 'a' contain the string 'b' within it, using a case-insensitive
 /// comparison?
 bool OIIO_API icontains (string_view a, string_view b);
@@ -368,7 +388,8 @@ string_view OIIO_API rstrip (string_view str, string_view chars=string_view());
 
 /// Fills the "result" list with the words in the string, using sep as
 /// the delimiter string.  If maxsplit is > -1, at most maxsplit splits
-/// are done. If sep is "", any whitespace string is a separator.
+/// are done. If sep is "", any whitespace string is a separator.  If the
+/// source `str` is empty, there will be zero pieces.
 void OIIO_API split (string_view str, std::vector<string_view> &result,
                      string_view sep = string_view(), int maxsplit = -1);
 void OIIO_API split (string_view str, std::vector<std::string> &result,
@@ -379,7 +400,8 @@ void OIIO_API split (string_view str, std::vector<std::string> &result,
 /// at most `maxsplit` split fragments will be produced (for example,
 /// maxsplit=2 will split at only the first separator, yielding at most two
 /// fragments). The result is returned as a vector of std::string (for
-/// `splits()`) or a vector of string_view (for `splitsv()`).
+/// `splits()`) or a vector of string_view (for `splitsv()`). If the source
+/// `str` is empty, there will be zero pieces.
 OIIO_API std::vector<std::string>
 splits (string_view str, string_view sep = "", int maxsplit = -1);
 OIIO_API std::vector<string_view>
